@@ -16,7 +16,7 @@
 //! Execution engine for EndBASIC programs.
 
 use crate::ast::{ArgSep, Expr, Statement, Value, VarRef, VarType};
-use crate::eval::{self, Vars};
+use crate::eval::{self, BuiltinFunction, CallableMetadata, CallableMetadataBuilder, Vars};
 use crate::parser::{self, Parser};
 use async_trait::async_trait;
 use std::collections::HashMap;
@@ -67,19 +67,16 @@ pub fn new_usage_error<T, S: Into<String>>(message: S) -> Result<T> {
 /// The commands themselves are immutable but they can reference mutable state.  Given that
 /// EndBASIC is not threaded, it is sufficient for those references to be behind a `RefCell`
 /// and/or an `Rc`.
+///
+/// Idiomatically, these objects need to provide a `new()` method that returns an `Rc<Callable>`, as
+/// that's the type used throughout the execution engine.
 #[async_trait(?Send)]
 pub trait BuiltinCommand {
-    /// Returns the name of the command, all in uppercase letters.
-    fn name(&self) -> &'static str;
-
-    /// Returns the name of the command's category.
-    fn category(&self) -> &'static str;
-
-    /// Returns a specification of the command's syntax.
-    fn syntax(&self) -> &'static str;
-
-    /// Returns the usage message of the command.
-    fn description(&self) -> &'static str;
+    /// Returns the metadata for this command.
+    ///
+    /// The return value takes the form of a reference to force the callable to store the metadata
+    /// as a struct field so that calls to this function are guaranteed to be cheap.
+    fn metadata(&self) -> &CallableMetadata;
 
     /// Executes the command.
     ///
@@ -96,48 +93,64 @@ pub trait BuiltinCommand {
 
 /// Builder pattern for a new machine.
 ///
-/// The `default` constructor creates a new builder for a machine with no builtin commands.
+/// The `default` constructor creates a new builder for a machine with no builtin commands and no
+/// builtin functions.
 #[derive(Default)]
 pub struct MachineBuilder {
-    builtins: HashMap<&'static str, Rc<dyn BuiltinCommand>>,
+    commands: HashMap<&'static str, Rc<dyn BuiltinCommand>>,
+    functions: HashMap<&'static str, Rc<dyn BuiltinFunction>>,
 }
 
 impl MachineBuilder {
     /// Registers the given builtin command, which must not yet be registered.
-    pub fn add_builtin(mut self, command: Rc<dyn BuiltinCommand>) -> Self {
+    pub fn add_command(mut self, command: Rc<dyn BuiltinCommand>) -> Self {
+        let metadata = command.metadata();
         assert!(
-            command.name() == command.name().to_ascii_uppercase(),
-            "Command name must be in uppercase"
-        );
-        assert!(
-            self.builtins.get(&command.name()).is_none(),
+            self.commands.get(&metadata.name()).is_none(),
             "Command with the same name already registered"
         );
-        for l in command.description().lines() {
-            assert!(!l.is_empty(), "Description cannot contain empty lines");
-        }
-        self.builtins.insert(command.name(), command);
+        self.commands.insert(metadata.name(), command);
         self
     }
 
-    /// Registers the given builtin commands.  See `add_builtin` for more details.
-    pub fn add_builtins(mut self, commands: Vec<Rc<dyn BuiltinCommand>>) -> Self {
+    /// Registers the given builtin function, which must not yet be registered.
+    pub fn add_function(mut self, function: Rc<dyn BuiltinFunction>) -> Self {
+        let metadata = function.metadata();
+        assert!(
+            self.functions.get(&metadata.name()).is_none(),
+            "Function with the same name already registered"
+        );
+        self.functions.insert(metadata.name(), function);
+        self
+    }
+
+    /// Registers the given builtin commands.  See `add_command` for more details.
+    pub fn add_commands(mut self, commands: Vec<Rc<dyn BuiltinCommand>>) -> Self {
         for command in commands {
-            self = self.add_builtin(command);
+            self = self.add_command(command);
+        }
+        self
+    }
+
+    /// Registers the given builtin functions.  See `add_function` for more details.
+    pub fn add_functions(mut self, functions: Vec<Rc<dyn BuiltinFunction>>) -> Self {
+        for function in functions {
+            self = self.add_function(function);
         }
         self
     }
 
     /// Creates a new machine with the current configuration.
     pub fn build(self) -> Machine {
-        Machine { builtins: self.builtins, vars: Vars::default() }
+        Machine { commands: self.commands, functions: self.functions, vars: Vars::default() }
     }
 }
 
 /// Executes an EndBASIC program and tracks its state.
 #[derive(Default)]
 pub struct Machine {
-    builtins: HashMap<&'static str, Rc<dyn BuiltinCommand>>,
+    commands: HashMap<&'static str, Rc<dyn BuiltinCommand>>,
+    functions: HashMap<&'static str, Rc<dyn BuiltinFunction>>,
     vars: Vars,
 }
 
@@ -147,9 +160,14 @@ impl Machine {
         self.vars.clear()
     }
 
-    /// Obtains immutable access to the builtins supported by this machine.
-    pub fn get_builtins(&self) -> &HashMap<&'static str, Rc<dyn BuiltinCommand>> {
-        &self.builtins
+    /// Obtains immutable access to the builtin commands provided by this machine.
+    pub fn get_commands(&self) -> &HashMap<&'static str, Rc<dyn BuiltinCommand>> {
+        &self.commands
+    }
+
+    /// Obtains immutable access to the builtin functions provided by this machine.
+    pub fn get_functions(&self) -> &HashMap<&'static str, Rc<dyn BuiltinFunction>> {
+        &self.functions
     }
 
     /// Obtains immutable access to the state of the variables.
@@ -191,7 +209,7 @@ impl Machine {
 
     /// Assigns the value of `expr` to the variable `vref`.
     fn assign(&mut self, vref: &VarRef, expr: &Expr) -> Result<()> {
-        let value = expr.eval(&self.vars)?;
+        let value = expr.eval(&self.vars, &self.functions)?;
         self.vars.set(&vref, value)?;
         Ok(())
     }
@@ -199,7 +217,7 @@ impl Machine {
     /// Executes an `IF` statement.
     async fn do_if(&mut self, branches: &[(Expr, Vec<Statement>)]) -> Result<()> {
         for (expr, stmts) in branches {
-            match expr.eval(&self.vars)? {
+            match expr.eval(&self.vars, &self.functions)? {
                 Value::Boolean(true) => {
                     for s in stmts {
                         self.exec_one(s).await?;
@@ -213,10 +231,46 @@ impl Machine {
         Ok(())
     }
 
+    /// Executes a `FOR` loop.
+    async fn do_for(
+        &mut self,
+        iterator: &VarRef,
+        start: &Expr,
+        end: &Expr,
+        next: &Expr,
+        body: &[Statement],
+    ) -> Result<()> {
+        debug_assert!(
+            iterator.ref_type() == VarType::Auto || iterator.ref_type() == VarType::Integer
+        );
+        let start_value = start.eval(&self.vars, &self.functions)?;
+        match start_value {
+            Value::Integer(_) => self.vars.set(iterator, start_value)?,
+            _ => return new_syntax_error("FOR supports integer iteration only"),
+        }
+
+        loop {
+            match end.eval(&self.vars, &self.functions)? {
+                Value::Boolean(false) => {
+                    break;
+                }
+                Value::Boolean(true) => (),
+                _ => panic!("Built-in condition should have evaluated to a boolean"),
+            }
+
+            for s in body {
+                self.exec_one(s).await?;
+            }
+
+            self.assign(iterator, next)?;
+        }
+        Ok(())
+    }
+
     /// Executes a `WHILE` loop.
     async fn do_while(&mut self, condition: &Expr, body: &[Statement]) -> Result<()> {
         loop {
-            match condition.eval(&self.vars)? {
+            match condition.eval(&self.vars, &self.functions)? {
                 Value::Boolean(true) => {
                     for s in body {
                         self.exec_one(s).await?;
@@ -234,7 +288,7 @@ impl Machine {
         match stmt {
             Statement::Assignment(vref, expr) => self.assign(vref, expr)?,
             Statement::BuiltinCall(name, args) => {
-                let cmd = match self.builtins.get(name.as_str()) {
+                let cmd = match self.commands.get(name.as_str()) {
                     Some(cmd) => cmd.clone(),
                     None => return new_syntax_error(format!("Unknown builtin {}", name)),
                 };
@@ -246,6 +300,13 @@ impl Machine {
                 let f: Pin<Box<dyn Future<Output = Result<()>>>> = Box::pin(self.do_if(branches));
                 f.await?;
             }
+            Statement::For(iterator, start, end, next, body) => {
+                // Change this to using FutureExt::boxed_local if we ever depend on the futures or
+                // futures_lite crate directly.
+                let f: Pin<Box<dyn Future<Output = Result<()>>>> =
+                    Box::pin(self.do_for(iterator, start, end, next, body));
+                f.await?;
+            }
             Statement::While(condition, body) => {
                 // Change this to using FutureExt::boxed_local if we ever depend on the futures or
                 // futures_lite crate directly.
@@ -271,25 +332,27 @@ impl Machine {
 }
 
 /// The `CLEAR` command.
-#[derive(Default)]
-pub struct ClearCommand {}
+pub struct ClearCommand {
+    metadata: CallableMetadata,
+}
+
+impl ClearCommand {
+    /// Creates a new `CLEAR` command that resets the state of the machine.
+    pub fn new() -> Rc<Self> {
+        Rc::from(Self {
+            metadata: CallableMetadataBuilder::new("CLEAR", VarType::Void)
+                .with_syntax("")
+                .with_category("Interpreter manipulation")
+                .with_description("Clears all variables to restore initial state.")
+                .build(),
+        })
+    }
+}
 
 #[async_trait(?Send)]
 impl BuiltinCommand for ClearCommand {
-    fn name(&self) -> &'static str {
-        "CLEAR"
-    }
-
-    fn category(&self) -> &'static str {
-        "Interpreter manipulation"
-    }
-
-    fn syntax(&self) -> &'static str {
-        ""
-    }
-
-    fn description(&self) -> &'static str {
-        "Clears all variables to restore initial state."
+    fn metadata(&self) -> &CallableMetadata {
+        &self.metadata
     }
 
     async fn exec(&self, args: &[(Option<Expr>, ArgSep)], machine: &mut Machine) -> Result<()> {
@@ -311,32 +374,26 @@ pub(crate) mod testutils {
     /// Every time this command is invoked, it yields the next value from the `data` iterator and
     /// assigns it to the variable provided as its only argument.
     pub(crate) struct InCommand {
+        metadata: CallableMetadata,
         data: Box<RefCell<dyn Iterator<Item = &'static &'static str>>>,
     }
 
     impl InCommand {
         /// Creates a new command with the golden `data`.
-        pub(crate) fn from(data: Box<RefCell<dyn Iterator<Item = &'static &'static str>>>) -> Self {
-            Self { data }
+        pub(crate) fn new(
+            data: Box<RefCell<dyn Iterator<Item = &'static &'static str>>>,
+        ) -> Rc<Self> {
+            Rc::from(Self {
+                metadata: CallableMetadataBuilder::new("IN", VarType::Void).test_build(),
+                data,
+            })
         }
     }
 
     #[async_trait(?Send)]
     impl BuiltinCommand for InCommand {
-        fn name(&self) -> &'static str {
-            "IN"
-        }
-
-        fn category(&self) -> &'static str {
-            "Testing"
-        }
-
-        fn syntax(&self) -> &'static str {
-            "variableref"
-        }
-
-        fn description(&self) -> &'static str {
-            "See docstring for test code."
+        fn metadata(&self) -> &CallableMetadata {
+            &self.metadata
         }
 
         async fn exec(&self, args: &[(Option<Expr>, ArgSep)], machine: &mut Machine) -> Result<()> {
@@ -364,39 +421,31 @@ pub(crate) mod testutils {
     /// This command only accepts arguments separated by the `;` short separator and concatenates
     /// them with a single space.
     pub(crate) struct OutCommand {
+        metadata: CallableMetadata,
         data: Rc<RefCell<Vec<String>>>,
     }
 
     impl OutCommand {
         /// Creates a new command that captures all calls into `data`.
-        pub(crate) fn from(data: Rc<RefCell<Vec<String>>>) -> Self {
-            Self { data }
+        pub(crate) fn new(data: Rc<RefCell<Vec<String>>>) -> Rc<Self> {
+            Rc::from(Self {
+                metadata: CallableMetadataBuilder::new("OUT", VarType::Void).test_build(),
+                data,
+            })
         }
     }
 
     #[async_trait(?Send)]
     impl BuiltinCommand for OutCommand {
-        fn name(&self) -> &'static str {
-            "OUT"
-        }
-
-        fn category(&self) -> &'static str {
-            "Testing"
-        }
-
-        fn syntax(&self) -> &'static str {
-            "[expr1 [; .. exprN]]"
-        }
-
-        fn description(&self) -> &'static str {
-            "See docstring for test code."
+        fn metadata(&self) -> &CallableMetadata {
+            &self.metadata
         }
 
         async fn exec(&self, args: &[(Option<Expr>, ArgSep)], machine: &mut Machine) -> Result<()> {
             let mut text = String::new();
             for arg in args.iter() {
                 if let Some(expr) = arg.0.as_ref() {
-                    text += &expr.eval(machine.get_vars())?.to_string();
+                    text += &expr.eval(machine.get_vars(), machine.get_functions())?.to_string();
                 }
                 match arg.1 {
                     ArgSep::End => break,
@@ -414,6 +463,7 @@ pub(crate) mod testutils {
 mod tests {
     use super::testutils::*;
     use super::*;
+    use crate::eval::testutils::*;
     use futures_lite::future::block_on;
     use std::cell::RefCell;
     use std::rc::Rc;
@@ -483,11 +533,10 @@ mod tests {
         golden_in: &'static [&'static str],
         captured_out: Rc<RefCell<Vec<String>>>,
     ) -> Result<()> {
-        let in_cmd = InCommand::from(Box::from(RefCell::from(golden_in.iter())));
-        let out_cmd = OutCommand::from(captured_out);
         let mut machine = MachineBuilder::default()
-            .add_builtin(Rc::from(in_cmd))
-            .add_builtin(Rc::from(out_cmd))
+            .add_command(InCommand::new(Box::from(RefCell::from(golden_in.iter()))))
+            .add_command(OutCommand::new(captured_out))
+            .add_function(SumFunction::new())
             .build();
         block_on(machine.exec(&mut input.as_bytes()))
     }
@@ -620,6 +669,109 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_for_incrementing() {
+        let code = r#"
+            IN first
+            IN last
+            FOR a = first TO last
+                OUT "a is"; a
+            NEXT
+        "#;
+        do_ok_test(code, &["0", "0"], &["a is 0"]);
+        do_ok_test(code, &["0", "3"], &["a is 0", "a is 1", "a is 2", "a is 3"]);
+    }
+
+    #[test]
+    fn test_for_incrementing_with_step() {
+        let code = r#"
+            IN first
+            IN last
+            FOR a = first TO last STEP 3
+                OUT "a is"; a
+            NEXT
+        "#;
+        do_ok_test(code, &["0", "0"], &["a is 0"]);
+        do_ok_test(code, &["0", "2"], &["a is 0"]);
+        do_ok_test(code, &["0", "3"], &["a is 0", "a is 3"]);
+        do_ok_test(code, &["0", "10"], &["a is 0", "a is 3", "a is 6", "a is 9"]);
+    }
+
+    #[test]
+    fn test_for_decrementing_with_step() {
+        let code = r#"
+            IN first
+            IN last
+            FOR a = first TO last STEP -2
+                OUT "a is"; a
+            NEXT
+        "#;
+        do_ok_test(code, &["0", "0"], &["a is 0"]);
+        do_ok_test(code, &["2", "0"], &["a is 2", "a is 0"]);
+        do_ok_test(code, &["-2", "-6"], &["a is -2", "a is -4", "a is -6"]);
+        do_ok_test(code, &["10", "1"], &["a is 10", "a is 8", "a is 6", "a is 4", "a is 2"]);
+    }
+
+    #[test]
+    fn test_for_already_done() {
+        do_ok_test("FOR i = 10 TO 9\nOUT i\nNEXT", &[], &[]);
+        do_ok_test("FOR i = 9 TO 10 STEP -1\nOUT i\nNEXT", &[], &[]);
+    }
+
+    #[test]
+    fn test_for_iterator_is_visible_after_next() {
+        let code = r#"
+            FOR something = 1 TO 10 STEP 8
+            NEXT
+            OUT something
+        "#;
+        do_ok_test(code, &[], &["17"]);
+    }
+
+    #[test]
+    fn test_for_iterator_can_be_modified() {
+        let code = r#"
+            FOR something = 1 TO 5
+                OUT something
+                something = something + 1
+            NEXT
+        "#;
+        do_ok_test(code, &[], &["1", "3", "5"]);
+    }
+
+    #[test]
+    fn test_for_errors() {
+        do_simple_error_test("FOR\nNEXT", "No iterator name in FOR statement");
+        do_simple_error_test("FOR a = 1 TO 10\nEND IF", "Unexpected token End in statement");
+
+        do_simple_error_test("FOR i = \"a\" TO 3\nNEXT", "FOR supports integer iteration only");
+        do_simple_error_test(
+            "FOR i = 1 TO \"a\"\nNEXT",
+            "Cannot compare Integer(1) and Text(\"a\") with <=",
+        );
+
+        do_simple_error_test(
+            "FOR i = \"b\" TO 7 STEP -8\nNEXT",
+            "FOR supports integer iteration only",
+        );
+        do_simple_error_test(
+            "FOR i = 1 TO \"b\" STEP -8\nNEXT",
+            "Cannot compare Integer(1) and Text(\"b\") with >=",
+        );
+
+        do_simple_error_test("FOR a = 1.0 TO 10.0\nNEXT", "FOR supports integer iteration only");
+    }
+
+    #[test]
+    fn test_function_call_ok() {
+        do_ok_test("x = 3\nOUT SUM(x, Sum%(4, 5), 1, sum())", &[], &["13"]);
+    }
+
+    #[test]
+    fn test_function_call_errors() {
+        do_simple_error_test("OUT SUM?()", "Incompatible type annotation for function call");
+    }
+
     #[test]
     fn test_while_ok() {
         let code = r#"
@@ -682,7 +834,7 @@ mod tests {
 
     #[test]
     fn test_clear_ok() {
-        let mut machine = MachineBuilder::default().add_builtin(Rc::from(ClearCommand {})).build();
+        let mut machine = MachineBuilder::default().add_command(ClearCommand::new()).build();
         block_on(machine.exec(&mut b"a = 1".as_ref())).unwrap();
         assert!(machine.get_var_as_int("a").is_ok());
         block_on(machine.exec(&mut b"CLEAR".as_ref())).unwrap();
@@ -691,7 +843,7 @@ mod tests {
 
     #[test]
     fn test_clear_errors() {
-        let mut machine = MachineBuilder::default().add_builtin(Rc::from(ClearCommand {})).build();
+        let mut machine = MachineBuilder::default().add_command(ClearCommand::new()).build();
         assert_eq!(
             "CLEAR takes no arguments",
             format!("{}", block_on(machine.exec(&mut b"CLEAR 123".as_ref())).unwrap_err())
