@@ -28,7 +28,9 @@ pub mod eval;
 pub mod exec;
 pub mod help;
 mod lexer;
+pub mod numerics;
 mod parser;
 pub mod program;
 mod reader;
 pub mod repl;
+pub mod strings;
