@@ -15,19 +15,21 @@
 
 //! Stored program manipulation and interactive editor.
 
-use crate::ast::{ArgSep, Expr, Value};
+use crate::ast::{ArgSep, Expr, Value, VarType};
 use crate::console::Console;
 use crate::editor::Editor;
+use crate::eval::{CallableMetadata, CallableMetadataBuilder};
 use crate::exec::{self, BuiltinCommand, Machine};
 use async_trait::async_trait;
 use std::cell::RefCell;
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, HashMap};
 use std::io;
 use std::path::PathBuf;
 use std::rc::Rc;
+use std::str;
 
 /// Metadata of an entry in the store.
-#[derive(Debug, Eq, PartialEq)]
+#[derive(Clone, Debug, Eq, PartialEq)]
 pub struct Metadata {
     /// Last modification time of the entry.
     pub date: time::OffsetDateTime,
@@ -51,6 +53,137 @@ pub trait Store {
     fn put(&mut self, name: &str, content: &str) -> io::Result<()>;
 }
 
+/// Wraps a `Store` and exposes a bunch of read-only demo files.
+///
+/// All demo file names are case insensitive.  However, this preserves the case sensitiveness
+/// behavior of the underlying store for any files that are passed through.
+///
+/// This takes ownership of any file names that start with `DEMO:`, which means any such files in
+/// the underlying store become invisible.  This should not be a problem in practice because most
+/// file systems deny the `:` character in file names.
+pub struct DemoStoreOverlay<S: Store> {
+    /// The demos to expose, expressed as a mapping of names to (metadata, content) pairs.
+    demos: HashMap<&'static str, (Metadata, String)>,
+
+    /// The wrapped store.
+    delegate: S,
+}
+
+/// Converts the raw bytes of a demo file into the program string to expose.
+///
+/// The input `bytes` must be valid UTF-8, which we can guarantee because these bytes come from
+/// files that we own in the source tree.
+///
+/// On Windows, the output string has all CRLF pairs converted to LF to ensure that the reported
+/// demo file sizes are consistent across platforms.
+fn process_demo(bytes: &[u8]) -> String {
+    let raw_content = str::from_utf8(bytes).expect("Malformed demo file");
+    if cfg!(target_os = "windows") {
+        raw_content.replace("\r\n", "\n")
+    } else {
+        raw_content.to_owned()
+    }
+}
+
+impl<S: Store> DemoStoreOverlay<S> {
+    /// Creates a new demo store that wraps the `delegate` store.
+    pub fn new(delegate: S) -> Self {
+        let mut demos = HashMap::default();
+        {
+            let content = process_demo(include_bytes!("../examples/guess.bas"));
+            let metadata = Metadata {
+                date: time::OffsetDateTime::from_unix_timestamp(1608693152),
+                length: content.len() as u64,
+            };
+            demos.insert("DEMO:GUESS.BAS", (metadata, content));
+        }
+        {
+            let content = process_demo(include_bytes!("../examples/hello.bas"));
+            let metadata = Metadata {
+                date: time::OffsetDateTime::from_unix_timestamp(1608646800),
+                length: content.len() as u64,
+            };
+            demos.insert("DEMO:HELLO.BAS", (metadata, content));
+        }
+        {
+            let content = process_demo(include_bytes!("../examples/tour.bas"));
+            let metadata = Metadata {
+                date: time::OffsetDateTime::from_unix_timestamp(1608774770),
+                length: content.len() as u64,
+            };
+            demos.insert("DEMO:TOUR.BAS", (metadata, content));
+        }
+        Self { demos, delegate }
+    }
+
+    /// Disowns and returns the underlying delegate store.
+    pub fn unmount(self) -> S {
+        self.delegate
+    }
+}
+
+impl<S: Store> Store for DemoStoreOverlay<S> {
+    fn delete(&mut self, name: &str) -> io::Result<()> {
+        let uc_name = name.to_ascii_uppercase();
+        match self.demos.get(&uc_name.as_ref()) {
+            Some(_) => {
+                Err(io::Error::new(io::ErrorKind::PermissionDenied, "Demo files are read-only"))
+            }
+            _ if uc_name.starts_with("DEMO:") => {
+                Err(io::Error::new(io::ErrorKind::PermissionDenied, "Demo files are read-only"))
+            }
+            _ => self.delegate.delete(name),
+        }
+    }
+
+    fn enumerate(&self) -> io::Result<BTreeMap<String, Metadata>> {
+        let mut entries = self.delegate.enumerate()?;
+
+        // TODO(https://github.com/rust-lang/rust/issues/70530): Use drain_filter when available.
+        let mut hidden_names = vec![];
+        for (name, _) in entries.iter() {
+            if name.to_ascii_uppercase().starts_with("DEMO:") {
+                hidden_names.push(name.to_owned());
+            }
+        }
+        for name in hidden_names {
+            entries.remove(&name);
+        }
+
+        for (name, (metadata, _content)) in self.demos.iter() {
+            entries.insert(name.to_string(), metadata.clone());
+        }
+        Ok(entries)
+    }
+
+    fn get(&self, name: &str) -> io::Result<String> {
+        let uc_name = name.to_ascii_uppercase();
+        match self.demos.get(&uc_name.as_ref()) {
+            Some(value) => {
+                let (_metadata, content) = value;
+                Ok(content.to_string())
+            }
+            _ if uc_name.starts_with("DEMO:") => {
+                Err(io::Error::new(io::ErrorKind::NotFound, "Non-existing demo file"))
+            }
+            _ => self.delegate.get(name),
+        }
+    }
+
+    fn put(&mut self, name: &str, content: &str) -> io::Result<()> {
+        let uc_name = name.to_ascii_uppercase();
+        match self.demos.get(&uc_name.as_ref()) {
+            Some(_) => {
+                Err(io::Error::new(io::ErrorKind::PermissionDenied, "Demo files are read-only"))
+            }
+            _ if uc_name.starts_with("DEMO:") => {
+                Err(io::Error::new(io::ErrorKind::PermissionDenied, "Demo files are read-only"))
+            }
+            _ => self.delegate.put(name, content),
+        }
+    }
+}
+
 /// Representation of the single program that we can keep in memory.
 #[async_trait(?Send)]
 pub trait Program {
@@ -123,27 +256,32 @@ fn show_dir(store: &dyn Store, console: &mut dyn Console) -> io::Result<()> {
 
 /// The `DEL` command.
 struct DelCommand {
+    metadata: CallableMetadata,
     store: Rc<RefCell<dyn Store>>,
 }
 
+impl DelCommand {
+    /// Creates a new `DEL` command that deletes a file from the `store`.
+    pub fn new(store: Rc<RefCell<dyn Store>>) -> Rc<Self> {
+        Rc::from(Self {
+            metadata: CallableMetadataBuilder::new("DEL", VarType::Void)
+                .with_syntax("filename")
+                .with_category("Stored program manipulation")
+                .with_description(
+                    "Deletes the given program.
+The filename must be a string and must be a basename (no directory components).  The .BAS \
+extension is optional, but if present, it must be .BAS.",
+                )
+                .build(),
+            store,
+        })
+    }
+}
+
 #[async_trait(?Send)]
 impl BuiltinCommand for DelCommand {
-    fn name(&self) -> &'static str {
-        "DEL"
-    }
-
-    fn category(&self) -> &'static str {
-        "Stored program manipulation"
-    }
-
-    fn syntax(&self) -> &'static str {
-        "filename"
-    }
-
-    fn description(&self) -> &'static str {
-        "Deletes the given program.
-The filename must be a string and must be a basename (no directory components).  The .BAS \
-extension is optional, but if present, it must be .BAS."
+    fn metadata(&self) -> &CallableMetadata {
+        &self.metadata
     }
 
     async fn exec(
@@ -155,7 +293,7 @@ extension is optional, but if present, it must be .BAS."
             return exec::new_usage_error("DEL requires a filename");
         }
         let arg0 = args[0].0.as_ref().expect("Single argument must be present");
-        match arg0.eval(machine.get_vars())? {
+        match arg0.eval(machine.get_vars(), machine.get_functions())? {
             Value::Text(t) => {
                 let name = to_filename(t)?;
                 self.store.borrow_mut().delete(&name)?;
@@ -168,26 +306,30 @@ extension is optional, but if present, it must be .BAS."
 
 /// The `DIR` command.
 struct DirCommand {
+    metadata: CallableMetadata,
     console: Rc<RefCell<dyn Console>>,
     store: Rc<RefCell<dyn Store>>,
 }
 
+impl DirCommand {
+    /// Creates a new `DIR` command that lists the contents of the `store` on the `console`.
+    pub fn new(console: Rc<RefCell<dyn Console>>, store: Rc<RefCell<dyn Store>>) -> Rc<Self> {
+        Rc::from(Self {
+            metadata: CallableMetadataBuilder::new("DIR", VarType::Void)
+                .with_syntax("")
+                .with_category("Stored program manipulation")
+                .with_description("Displays the list of files on disk.")
+                .build(),
+            console,
+            store,
+        })
+    }
+}
+
 #[async_trait(?Send)]
 impl BuiltinCommand for DirCommand {
-    fn name(&self) -> &'static str {
-        "DIR"
-    }
-
-    fn category(&self) -> &'static str {
-        "Stored program manipulation"
-    }
-
-    fn syntax(&self) -> &'static str {
-        ""
-    }
-
-    fn description(&self) -> &'static str {
-        "Displays the list of files on disk."
+    fn metadata(&self) -> &CallableMetadata {
+        &self.metadata
     }
 
     async fn exec(
@@ -205,26 +347,30 @@ impl BuiltinCommand for DirCommand {
 
 /// The `EDIT` command.
 struct EditCommand {
+    metadata: CallableMetadata,
     console: Rc<RefCell<dyn Console>>,
     program: Rc<RefCell<dyn Program>>,
 }
 
+impl EditCommand {
+    /// Creates a new `EDIT` command that edits the stored `program` in the `console`.
+    pub fn new(console: Rc<RefCell<dyn Console>>, program: Rc<RefCell<dyn Program>>) -> Rc<Self> {
+        Rc::from(Self {
+            metadata: CallableMetadataBuilder::new("EDIT", VarType::Void)
+                .with_syntax("")
+                .with_category("Stored program manipulation")
+                .with_description("Interactively edits the stored program.")
+                .build(),
+            console,
+            program,
+        })
+    }
+}
+
 #[async_trait(?Send)]
 impl BuiltinCommand for EditCommand {
-    fn name(&self) -> &'static str {
-        "EDIT"
-    }
-
-    fn category(&self) -> &'static str {
-        "Stored program manipulation"
-    }
-
-    fn syntax(&self) -> &'static str {
-        "[lineno%]"
-    }
-
-    fn description(&self) -> &'static str {
-        "Interactively edits the stored program."
+    fn metadata(&self) -> &CallableMetadata {
+        &self.metadata
     }
 
     async fn exec(
@@ -245,28 +391,34 @@ impl BuiltinCommand for EditCommand {
 
 /// The `LOAD` command.
 struct LoadCommand {
+    metadata: CallableMetadata,
     store: Rc<RefCell<dyn Store>>,
     program: Rc<RefCell<dyn Program>>,
 }
 
+impl LoadCommand {
+    /// Creates a new `LOAD` command that loads a program from the `store` into `program`.
+    pub fn new(store: Rc<RefCell<dyn Store>>, program: Rc<RefCell<dyn Program>>) -> Rc<Self> {
+        Rc::from(Self {
+            metadata: CallableMetadataBuilder::new("LOAD", VarType::Void)
+                .with_syntax("filename")
+                .with_category("Stored program manipulation")
+                .with_description(
+                    "Loads the given program.
+The filename must be a string and must be a basename (no directory components).  The .BAS \
+extension is optional, but if present, it must be .BAS.",
+                )
+                .build(),
+            store,
+            program,
+        })
+    }
+}
+
 #[async_trait(?Send)]
 impl BuiltinCommand for LoadCommand {
-    fn name(&self) -> &'static str {
-        "LOAD"
-    }
-
-    fn category(&self) -> &'static str {
-        "Stored program manipulation"
-    }
-
-    fn syntax(&self) -> &'static str {
-        "filename"
-    }
-
-    fn description(&self) -> &'static str {
-        "Loads the given program.
-The filename must be a string and must be a basename (no directory components).  The .BAS \
-extension is optional, but if present, it must be .BAS."
+    fn metadata(&self) -> &CallableMetadata {
+        &self.metadata
     }
 
     async fn exec(
@@ -278,7 +430,7 @@ extension is optional, but if present, it must be .BAS."
             return exec::new_usage_error("LOAD requires a filename");
         }
         let arg0 = args[0].0.as_ref().expect("Single argument must be present");
-        match arg0.eval(machine.get_vars())? {
+        match arg0.eval(machine.get_vars(), machine.get_functions())? {
             Value::Text(t) => {
                 let name = to_filename(t)?;
                 let content = self.store.borrow().get(&name)?;
@@ -293,25 +445,28 @@ extension is optional, but if present, it must be .BAS."
 
 /// The `NEW` command.
 struct NewCommand {
+    metadata: CallableMetadata,
     program: Rc<RefCell<dyn Program>>,
 }
 
+impl NewCommand {
+    /// Creates a new `NEW` command that clears the contents of `program`.
+    pub fn new(program: Rc<RefCell<dyn Program>>) -> Rc<Self> {
+        Rc::from(Self {
+            metadata: CallableMetadataBuilder::new("NEW", VarType::Void)
+                .with_syntax("")
+                .with_category("Stored program manipulation")
+                .with_description("Clears the stored program from memory.")
+                .build(),
+            program,
+        })
+    }
+}
+
 #[async_trait(?Send)]
 impl BuiltinCommand for NewCommand {
-    fn name(&self) -> &'static str {
-        "NEW"
-    }
-
-    fn category(&self) -> &'static str {
-        "Stored program manipulation"
-    }
-
-    fn syntax(&self) -> &'static str {
-        ""
-    }
-
-    fn description(&self) -> &'static str {
-        "Clears the stored program from memory."
+    fn metadata(&self) -> &CallableMetadata {
+        &self.metadata
     }
 
     async fn exec(
@@ -330,27 +485,32 @@ impl BuiltinCommand for NewCommand {
 
 /// The `RUN` command.
 struct RunCommand {
+    metadata: CallableMetadata,
     program: Rc<RefCell<dyn Program>>,
 }
 
+impl RunCommand {
+    /// Creates a new `RUN` command that executes the `program`.
+    pub fn new(program: Rc<RefCell<dyn Program>>) -> Rc<Self> {
+        Rc::from(Self {
+            metadata: CallableMetadataBuilder::new("RUN", VarType::Void)
+                .with_syntax("")
+                .with_category("Stored program manipulation")
+                .with_description(
+                    "Runs the stored program.
+Note that the program runs in the context of the interpreter so it will pick up any variables \
+and other state that may already be set.",
+                )
+                .build(),
+            program,
+        })
+    }
+}
+
 #[async_trait(?Send)]
 impl BuiltinCommand for RunCommand {
-    fn name(&self) -> &'static str {
-        "RUN"
-    }
-
-    fn category(&self) -> &'static str {
-        "Stored program manipulation"
-    }
-
-    fn syntax(&self) -> &'static str {
-        ""
-    }
-
-    fn description(&self) -> &'static str {
-        "Runs the stored program.
-Note that the program runs in the context of the interpreter so it will pick up any variables \
-and other state that may already be set."
+    fn metadata(&self) -> &CallableMetadata {
+        &self.metadata
     }
 
     async fn exec(
@@ -368,28 +528,34 @@ and other state that may already be set."
 
 /// The `SAVE` command.
 struct SaveCommand {
+    metadata: CallableMetadata,
     store: Rc<RefCell<dyn Store>>,
     program: Rc<RefCell<dyn Program>>,
 }
 
+impl SaveCommand {
+    /// Creates a new `SAVE` command that saves the contents of the `program` in the `store`.
+    pub fn new(store: Rc<RefCell<dyn Store>>, program: Rc<RefCell<dyn Program>>) -> Rc<Self> {
+        Rc::from(Self {
+            metadata: CallableMetadataBuilder::new("SAVE", VarType::Void)
+                .with_syntax("filename")
+                .with_category("Stored program manipulation")
+                .with_description(
+                    "Saves the current program in memory to the given filename.
+The filename must be a string and must be a basename (no directory components).  The .BAS \
+extension is optional, but if present, it must be .BAS.",
+                )
+                .build(),
+            store,
+            program,
+        })
+    }
+}
+
 #[async_trait(?Send)]
 impl BuiltinCommand for SaveCommand {
-    fn name(&self) -> &'static str {
-        "SAVE"
-    }
-
-    fn category(&self) -> &'static str {
-        "Stored program manipulation"
-    }
-
-    fn syntax(&self) -> &'static str {
-        "filename"
-    }
-
-    fn description(&self) -> &'static str {
-        "Saves the current program in memory to the given filename.
-The filename must be a string and must be a basename (no directory components).  The .BAS \
-extension is optional, but if present, it must be .BAS."
+    fn metadata(&self) -> &CallableMetadata {
+        &self.metadata
     }
 
     async fn exec(
@@ -401,7 +567,7 @@ extension is optional, but if present, it must be .BAS."
             return exec::new_usage_error("SAVE requires a filename");
         }
         let arg0 = args[0].0.as_ref().expect("Single argument must be present");
-        match arg0.eval(machine.get_vars())? {
+        match arg0.eval(machine.get_vars(), machine.get_functions())? {
             Value::Text(t) => {
                 let name = to_filename(t)?;
                 let content = self.program.borrow().text();
@@ -421,13 +587,13 @@ fn all_commands_for(
     store: Rc<RefCell<dyn Store>>,
 ) -> Vec<Rc<dyn BuiltinCommand>> {
     vec![
-        Rc::from(DelCommand { store: store.clone() }),
-        Rc::from(DirCommand { console: console.clone(), store: store.clone() }),
-        Rc::from(EditCommand { console: console.clone(), program: program.clone() }),
-        Rc::from(LoadCommand { store: store.clone(), program: program.clone() }),
-        Rc::from(NewCommand { program: program.clone() }),
-        Rc::from(RunCommand { program: program.clone() }),
-        Rc::from(SaveCommand { store: store.clone(), program }),
+        DelCommand::new(store.clone()),
+        DirCommand::new(console.clone(), store.clone()),
+        EditCommand::new(console.clone(), program.clone()),
+        LoadCommand::new(store.clone(), program.clone()),
+        NewCommand::new(program.clone()),
+        RunCommand::new(program.clone()),
+        SaveCommand::new(store, program),
     ]
 }
 
@@ -520,6 +686,111 @@ mod tests {
     use crate::exec::MachineBuilder;
     use futures_lite::future::block_on;
 
+    #[test]
+    fn test_demo_store_overlay_delete() {
+        let mut store = InMemoryStore::default();
+        store.put("delete.bas", "underlying file").unwrap();
+        store.put("keep.bas", "underlying file").unwrap();
+        store.put("demo:unknown.bas", "should not be touched").unwrap();
+        let store = {
+            let mut store = DemoStoreOverlay::new(store);
+
+            store.delete("delete.bas").unwrap();
+            assert_eq!(io::ErrorKind::NotFound, store.delete("KEEP.Bas").unwrap_err().kind());
+
+            assert_eq!(
+                io::ErrorKind::PermissionDenied,
+                store.delete("demo:hello.bas").unwrap_err().kind()
+            );
+            assert_eq!(
+                io::ErrorKind::PermissionDenied,
+                store.delete("DEMO:Hello.BAS").unwrap_err().kind()
+            );
+
+            assert_eq!(
+                io::ErrorKind::PermissionDenied,
+                store.delete("demo:unknown.bas").unwrap_err().kind()
+            );
+
+            store.unmount()
+        };
+        assert_eq!(io::ErrorKind::NotFound, store.get("delete.bas").unwrap_err().kind());
+        assert_eq!("underlying file", store.get("keep.bas").unwrap());
+        assert_eq!(io::ErrorKind::NotFound, store.get("demo:hello.bas").unwrap_err().kind());
+        assert_eq!("should not be touched", store.get("demo:unknown.bas").unwrap());
+    }
+
+    #[test]
+    fn test_demo_store_overlay_enumerate() {
+        let mut store = InMemoryStore::default();
+        store.put("under.bas", "underlying file").unwrap();
+        store.put("demo:hidden.bas", "will be clobbered").unwrap();
+        let store = DemoStoreOverlay::new(store);
+
+        let entries = store.enumerate().unwrap();
+        assert!(entries.contains_key("under.bas"));
+        assert!(entries.contains_key("DEMO:GUESS.BAS"));
+        assert!(entries.contains_key("DEMO:HELLO.BAS"));
+        assert!(entries.contains_key("DEMO:TOUR.BAS"));
+        assert!(!entries.contains_key("DEMO:HIDDEN.BAS"));
+        assert!(!entries.contains_key("demo:hidden.bas"));
+    }
+
+    #[test]
+    fn test_demo_store_overlay_get() {
+        let mut store = InMemoryStore::default();
+        store.put("under.bas", "underlying file").unwrap();
+        store.put("demo:hidden.bas", "will be clobbered").unwrap();
+        let store = DemoStoreOverlay::new(store);
+
+        assert_eq!("underlying file", store.get("under.bas").unwrap());
+        assert_eq!(io::ErrorKind::NotFound, store.get("Under.bas").unwrap_err().kind());
+
+        assert_eq!(
+            process_demo(include_bytes!("../examples/hello.bas")),
+            store.get("demo:hello.bas").unwrap()
+        );
+        assert_eq!(
+            process_demo(include_bytes!("../examples/hello.bas")),
+            store.get("Demo:Hello.Bas").unwrap()
+        );
+
+        assert_eq!(io::ErrorKind::NotFound, store.get("demo:hidden.bas").unwrap_err().kind());
+        assert_eq!(io::ErrorKind::NotFound, store.get("demo:unknown.bas").unwrap_err().kind());
+        assert_eq!(io::ErrorKind::NotFound, store.get("unknown.bas").unwrap_err().kind());
+    }
+
+    #[test]
+    fn test_demo_store_overlay_put() {
+        let mut store = InMemoryStore::default();
+        store.put("modify.bas", "previous contents").unwrap();
+        store.put("avoid.bas", "previous contents").unwrap();
+        let store = {
+            let mut store = DemoStoreOverlay::new(store);
+
+            store.put("modify.bas", "new contents").unwrap();
+
+            assert_eq!(
+                io::ErrorKind::PermissionDenied,
+                store.put("demo:hello.bas", "").unwrap_err().kind()
+            );
+            assert_eq!(
+                io::ErrorKind::PermissionDenied,
+                store.put("DEMO:Hello.BAS", "").unwrap_err().kind()
+            );
+
+            assert_eq!(
+                io::ErrorKind::PermissionDenied,
+                store.put("demo:unknown.bas", "").unwrap_err().kind()
+            );
+
+            store.unmount()
+        };
+        assert_eq!(io::ErrorKind::NotFound, store.get("demo:unknown.bas").unwrap_err().kind());
+        assert_eq!("new contents", store.get("modify.bas").unwrap());
+        assert_eq!("previous contents", store.get("avoid.bas").unwrap());
+    }
+
     /// Runs the `input` code on a new machine that stores programs in `store` and verifies its
     /// output.
     ///
@@ -539,7 +810,7 @@ mod tests {
         let console =
             Rc::from(RefCell::from(MockConsoleBuilder::new().add_input_chars(golden_in).build()));
         let mut machine = MachineBuilder::default()
-            .add_builtins(all_commands_for(program.clone(), console.clone(), store))
+            .add_commands(all_commands_for(program.clone(), console.clone(), store))
             .build();
         block_on(machine.exec(&mut input.as_bytes())).expect("Execution failed");
         let expected_out: Vec<CapturedOut> =
@@ -567,7 +838,7 @@ mod tests {
         let console = Rc::from(RefCell::from(MockConsoleBuilder::new().build()));
         let program = Rc::from(RefCell::from(RecordedProgram::new("")));
         let mut machine = MachineBuilder::default()
-            .add_builtins(all_commands_for(program, console.clone(), store))
+            .add_commands(all_commands_for(program, console.clone(), store))
             .build();
         assert_eq!(
             expected_err,
@@ -754,13 +1025,14 @@ mod tests {
     fn test_new_clears_program_and_variables() {
         let program = Rc::from(RefCell::from(RecordedProgram::new("some stuff")));
 
-        let mut machine = MachineBuilder::default()
-            .add_builtin(Rc::from(NewCommand { program: program.clone() }))
-            .build();
+        let mut machine =
+            MachineBuilder::default().add_command(NewCommand::new(program.clone())).build();
 
         block_on(machine.exec(&mut b"NEW".as_ref())).unwrap();
         assert!(program.borrow().text().is_empty());
         assert!(machine.get_vars().is_empty());
+        // TODO(jmmv): If we get user-supplied functions, we need to check here that they were
+        // cleared too.
     }
 
     #[test]
@@ -778,10 +1050,9 @@ mod tests {
         let program = Rc::from(RefCell::from(RecordedProgram::new("OUT var\nvar = var + 1")));
 
         let captured_out = Rc::from(RefCell::from(vec![]));
-        let out_cmd = OutCommand::from(captured_out.clone());
         let mut machine = MachineBuilder::default()
-            .add_builtin(Rc::from(out_cmd))
-            .add_builtin(Rc::from(RunCommand { program }))
+            .add_command(OutCommand::new(captured_out.clone()))
+            .add_command(RunCommand::new(program))
             .build();
 
         block_on(machine.exec(&mut b"var = 7: RUN".as_ref())).unwrap();
