@@ -15,11 +15,14 @@
 
 //! Interactive interpreter for the EndBASIC language.
 
-use crate::ast::{ArgSep, Expr, Value};
+use crate::ast::{ArgSep, Expr, Value, VarType};
 use crate::console::{self, Console};
+use crate::eval::{CallableMetadata, CallableMetadataBuilder};
 use crate::exec::{self, BuiltinCommand, ClearCommand, Machine, MachineBuilder};
 use crate::help::HelpCommand;
+use crate::numerics;
 use crate::program::{self, Store};
+use crate::strings;
 use async_trait::async_trait;
 use std::cell::RefCell;
 use std::io;
@@ -27,33 +30,31 @@ use std::rc::Rc;
 
 /// The `EXIT` command.
 pub struct ExitCommand {
+    metadata: CallableMetadata,
     code: Rc<RefCell<Option<i32>>>,
 }
 
 impl ExitCommand {
     /// Creates a new command that updates `code` with the exit code once called.
-    pub fn new(code: Rc<RefCell<Option<i32>>>) -> Self {
-        Self { code }
+    pub fn new(code: Rc<RefCell<Option<i32>>>) -> Rc<Self> {
+        Rc::from(Self {
+            metadata: CallableMetadataBuilder::new("EXIT", VarType::Void)
+                .with_syntax("[code%]")
+                .with_category("Interpreter manipulation")
+                .with_description(
+                    "Exits the interpreter.
+The optional code indicates the return value to return to the system.",
+                )
+                .build(),
+            code,
+        })
     }
 }
 
 #[async_trait(?Send)]
 impl BuiltinCommand for ExitCommand {
-    fn name(&self) -> &'static str {
-        "EXIT"
-    }
-
-    fn category(&self) -> &'static str {
-        "Interpreter manipulation"
-    }
-
-    fn syntax(&self) -> &'static str {
-        "[code%]"
-    }
-
-    fn description(&self) -> &'static str {
-        "Exits the interpreter.
-The optional code indicates the return value to return to the system."
+    fn metadata(&self) -> &CallableMetadata {
+        &self.metadata
     }
 
     async fn exec(
@@ -63,18 +64,20 @@ The optional code indicates the return value to return to the system."
     ) -> exec::Result<()> {
         let arg = match args {
             [] => 0,
-            [(Some(expr), ArgSep::End)] => match expr.eval(machine.get_vars())? {
-                Value::Integer(n) => {
-                    if n < 0 {
-                        return exec::new_usage_error("Exit code must be a positive integer");
+            [(Some(expr), ArgSep::End)] => {
+                match expr.eval(machine.get_vars(), machine.get_functions())? {
+                    Value::Integer(n) => {
+                        if n < 0 {
+                            return exec::new_usage_error("Exit code must be a positive integer");
+                        }
+                        if n >= 128 {
+                            return exec::new_usage_error("Exit code cannot be larger than 127");
+                        }
+                        n
                     }
-                    if n >= 128 {
-                        return exec::new_usage_error("Exit code cannot be larger than 127");
-                    }
-                    n
+                    _ => return exec::new_usage_error("Exit code must be a positive integer"),
                 }
-                _ => return exec::new_usage_error("Exit code must be a positive integer"),
-            },
+            }
             _ => return exec::new_usage_error("EXIT takes zero or one argument"),
         };
         let mut code = self.code.borrow_mut();
@@ -95,29 +98,27 @@ The optional code indicates the return value to return to the system."
 pub async fn run_repl_loop(
     console: Rc<RefCell<dyn Console>>,
     store: Rc<RefCell<dyn Store>>,
-    banner: &[&str],
 ) -> io::Result<i32> {
     let exit_code = Rc::from(RefCell::from(None));
-    let mut machine = MachineBuilder::default()
-        .add_builtin(Rc::from(ClearCommand::default()))
-        .add_builtin(Rc::from(ExitCommand::new(exit_code.clone())))
-        .add_builtin(Rc::from(HelpCommand::new(console.clone())))
-        .add_builtins(console::all_commands(console.clone()))
-        .add_builtins(program::all_commands(console.clone(), store))
-        .build();
+    let mut machine = {
+        let mut builder = MachineBuilder::default()
+            .add_command(ClearCommand::new())
+            .add_command(ExitCommand::new(exit_code.clone()))
+            .add_command(HelpCommand::new(console.clone()))
+            .add_commands(console::all_commands(console.clone()))
+            .add_commands(program::all_commands(console.clone(), store))
+            .add_functions(strings::all_functions());
+        builder = numerics::add_all(builder);
+        builder.build()
+    };
 
     {
         let mut console = console.borrow_mut();
         console.print("")?;
         console.print(&format!("    Welcome to EndBASIC {}.", env!("CARGO_PKG_VERSION")))?;
-        if !banner.is_empty() {
-            console.print("")?;
-            for line in banner {
-                console.print(&format!("    {}", line))?;
-            }
-            console.print("")?;
-        }
+        console.print("")?;
         console.print("    Type HELP for interactive usage information.")?;
+        console.print("    Type LOAD \"DEMO:TOUR.BAS\": RUN for a guided tour.")?;
         console.print("")?;
     }
 
@@ -172,9 +173,8 @@ mod tests {
     /// Runs the code `input` and expects it to fail with `expected_err`.
     fn do_error_test(input: &str, expected_err: &str) {
         let exit_code = Rc::from(RefCell::from(None));
-        let mut machine = MachineBuilder::default()
-            .add_builtin(Rc::from(ExitCommand::new(exit_code.clone())))
-            .build();
+        let mut machine =
+            MachineBuilder::default().add_command(ExitCommand::new(exit_code.clone())).build();
         let err = block_on(machine.exec(&mut input.as_bytes())).unwrap_err();
         assert_eq!(expected_err, format!("{}", err));
         assert!(exit_code.borrow().is_none());
@@ -183,9 +183,8 @@ mod tests {
     #[test]
     fn test_exit_no_code() {
         let exit_code = Rc::from(RefCell::from(None));
-        let mut machine = MachineBuilder::default()
-            .add_builtin(Rc::from(ExitCommand::new(exit_code.clone())))
-            .build();
+        let mut machine =
+            MachineBuilder::default().add_command(ExitCommand::new(exit_code.clone())).build();
         block_on(machine.exec(&mut b"a = 3: EXIT: a = 4".as_ref())).unwrap_err();
         assert_eq!(0, exit_code.borrow().unwrap());
         assert_eq!(3, machine.get_var_as_int("a").unwrap());
@@ -193,9 +192,8 @@ mod tests {
 
     fn do_exit_with_code_test(code: i32) {
         let exit_code = Rc::from(RefCell::from(None));
-        let mut machine = MachineBuilder::default()
-            .add_builtin(Rc::from(ExitCommand::new(exit_code.clone())))
-            .build();
+        let mut machine =
+            MachineBuilder::default().add_command(ExitCommand::new(exit_code.clone())).build();
         block_on(machine.exec(&mut format!("a = 3: EXIT {}: a = 4", code).as_bytes())).unwrap_err();
         assert_eq!(code, exit_code.borrow().unwrap());
         assert_eq!(3, machine.get_var_as_int("a").unwrap());
